@@ -30,19 +30,16 @@
 #include <string.h>
 #include <stdlib.h>
 #include <gtk/gtk.h>
+#include <gio/gio.h>
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagemanager.h>
 #include <gtksourceview/gtksourcestyleschememanager.h>
 #include <gtksourceview/gtksourceprintcompositor.h>
+#include <gtksourceview/gtksourceiter.h>
 #ifdef TEST_XML_MEM
 #include <libxml/xmlreader.h>
 #endif
-#ifdef USE_GNOME_VFS
-#include <libgnomevfs/gnome-vfs-init.h>
-#include <libgnomevfs/gnome-vfs-mime-utils.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-#endif
 
 /* Global list of open windows */
 
@@ -62,8 +59,10 @@ static GtkSourceStyleScheme *style_scheme = NULL;
 static void       open_file_cb                   (GtkAction       *action,
 						  gpointer         user_data);
 static void       print_file_cb                  (GtkAction       *action,
-						  gpointer         user_data);						  
-static void       debug_thing_3_cb		 (GtkAction       *action,
+						  gpointer         user_data);
+static void       find_cb			 (GtkAction       *action,
+						  gpointer         user_data);
+static void       replace_cb			 (GtkAction       *action,
 						  gpointer         user_data);
 
 static void       new_view_cb                    (GtkAction       *action,
@@ -74,8 +73,12 @@ static void       marks_toggled_cb               (GtkAction       *action,
 						  gpointer         user_data);
 static void       margin_toggled_cb              (GtkAction       *action,
 						  gpointer         user_data);
+static void       hl_bracket_toggled_cb          (GtkAction       *action,
+						  gpointer         user_data);
 static void       hl_line_toggled_cb             (GtkAction       *action,
 						  gpointer         user_data);
+static void       draw_spaces_toggled_cb	 (GtkAction       *action,
+						  gpointer         user_data);
 static void       wrap_lines_toggled_cb          (GtkAction       *action,
 						  gpointer         user_data);
 static void       auto_indent_toggled_cb         (GtkAction       *action,
@@ -112,11 +115,16 @@ static GtkActionEntry view_action_entries[] = {
 	{ "TabWidth", NULL, "_Tab Width" },
 	{ "IndentWidth", NULL, "I_ndent Width" },
 	{ "SmartHomeEnd", NULL, "_Smart Home/End" },
-	{ "DebugThing3", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
-	  "Search and Replace", G_CALLBACK (debug_thing_3_cb) },
+	{ "Find", GTK_STOCK_FIND, "_Find", "<control>F",
+	  "Find", G_CALLBACK (find_cb) },
+	{ "Replace", GTK_STOCK_FIND_AND_REPLACE, "Search and _Replace", "<control>R",
+	  "Search and Replace", G_CALLBACK (replace_cb) },
 };
 
 static GtkToggleActionEntry toggle_entries[] = {
+	{ "HlBracket", NULL, "Highlight Matching _Bracket", NULL,
+	  "Toggle highlighting of matching bracket",
+	  G_CALLBACK (hl_bracket_toggled_cb), FALSE },
 	{ "ShowNumbers", NULL, "Show _Line Numbers", NULL,
 	  "Toggle visibility of line numbers in the left margin",
 	  G_CALLBACK (numbers_toggled_cb), FALSE },
@@ -129,6 +137,9 @@ static GtkToggleActionEntry toggle_entries[] = {
 	{ "HlLine", NULL, "_Highlight Current Line", NULL,
 	  "Toggle highlighting of current line",
 	  G_CALLBACK (hl_line_toggled_cb), FALSE },
+	{ "DrawSpaces", NULL, "_Draw Spaces", NULL,
+	  "Draw Spaces",
+	  G_CALLBACK (draw_spaces_toggled_cb), FALSE },
 	{ "WrapLines", NULL, "_Wrap Lines", NULL,
 	  "Toggle line wrapping",
 	  G_CALLBACK (wrap_lines_toggled_cb), FALSE },
@@ -179,10 +190,12 @@ static const gchar *view_ui_description =
 "    <menu action=\"ViewMenu\">"
 "      <menuitem action=\"NewView\"/>"
 "      <separator/>"
+"      <menuitem action=\"HlBracket\"/>"
 "      <menuitem action=\"ShowNumbers\"/>"
 "      <menuitem action=\"ShowMarks\"/>"
 "      <menuitem action=\"ShowMargin\"/>"
 "      <menuitem action=\"HlLine\"/>"
+"      <menuitem action=\"DrawSpaces\"/>"
 "      <menuitem action=\"WrapLines\"/>"
 "      <separator/>"
 "      <menuitem action=\"AutoIndent\"/>"
@@ -220,7 +233,8 @@ static const gchar *buffer_ui_description =
 "    <menu action=\"FileMenu\">"
 "      <menuitem action=\"Open\"/>"
 "      <menuitem action=\"Print\"/>"
-"      <menuitem action=\"DebugThing3\"/>"
+"      <menuitem action=\"Find\"/>"
+"      <menuitem action=\"Replace\"/>"
 "      <separator/>"
 "      <menuitem action=\"Quit\"/>"
 "    </menu>"
@@ -305,129 +319,45 @@ remove_all_marks (GtkSourceBuffer *buffer)
 	gtk_source_buffer_remove_source_marks (buffer, &s, &e, NULL);
 }
 
-/* Note this is wrong for several reasons, e.g. g_pattern_match is broken
- * for glob matching. */
 static GtkSourceLanguage *
-get_language_for_filename (const gchar *filename)
+get_language_for_file (GtkTextBuffer *buffer, const gchar *filename)
 {
-	const gchar * const *languages;
-	gchar *filename_utf8;
 	GtkSourceLanguageManager *manager;
+	GtkSourceLanguage *language;
+	GtkTextIter start, end;
+	gchar *text;
+	gchar *content_type;
+	gboolean result_uncertain;
 
-	filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
-	g_return_val_if_fail (filename_utf8 != NULL, NULL);
-
-	manager = gtk_source_language_manager_get_default ();
-	languages = gtk_source_language_manager_get_language_ids (manager);
-
-	while (*languages != NULL)
-	{
-		GtkSourceLanguage *lang;
-		gchar **globs, **p;
-
-		lang = gtk_source_language_manager_get_language (manager,
-								 *languages);
-		g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (lang), NULL);
-		++languages;
-
-		globs = gtk_source_language_get_globs (lang);
-		if (globs == NULL)
-			continue;
-
-		for (p = globs; *p != NULL; p++)
-		{
-			if (g_pattern_match_simple (*p, filename_utf8))
-			{
-				g_strfreev (globs);
-				g_free (filename_utf8);
-
-				return lang;
-			}
-		}
-
-		g_strfreev (globs);
-	}
-
-	g_free (filename_utf8);
-	return NULL;
-}
-
-/* Note this is wrong, because it ignores mime parent types and subtypes.
- * It's fine to use in a simplish program like this, but is unacceptable
- * in a serious text editor. */
-static GtkSourceLanguage *
-get_language_for_mime_type (const gchar *mime)
-{
-	const gchar * const *languages;
-	GtkSourceLanguageManager *manager;
-
-	manager = gtk_source_language_manager_get_default ();
-	languages = gtk_source_language_manager_get_language_ids (manager);
-
-	while (*languages != NULL)
-	{
-		GtkSourceLanguage *lang;
-		gchar **mimetypes, **p;
-
-		lang = gtk_source_language_manager_get_language (manager,
-								 *languages);
-		g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (lang), NULL);
-		++languages;
-
-		mimetypes = gtk_source_language_get_mime_types (lang);
-
-		if (mimetypes == NULL)
-			continue;
-
-		for (p = mimetypes; *p != NULL; p++)
-		{
-			if (strcmp (*p, mime) == 0)
-			{
-				g_strfreev (mimetypes);
-				return lang;
-			}
-		}
-
-		g_strfreev (mimetypes);
-	}
-
-	return NULL;
-}
-
-static GtkSourceLanguage *
-get_language_for_file (const gchar *filename)
-{
-	GtkSourceLanguage *language = NULL;
-
-#if defined(USE_GNOME_VFS)
-	gchar *mime_type;
-	gchar *uri;
-
-	/* I hate this! */
-	if (g_path_is_absolute (filename))
-	{
-		uri = gnome_vfs_get_uri_from_local_path (filename);
-	}
+	gtk_text_buffer_get_start_iter (buffer, &start);
+	if (gtk_text_buffer_get_char_count (buffer) < 1024)
+		gtk_text_buffer_get_end_iter (buffer, &end);
 	else
+		gtk_text_buffer_get_iter_at_offset (buffer, &end, 1024);
+	text = gtk_text_buffer_get_slice (buffer, &start, &end, TRUE);
+
+	content_type = g_content_type_guess (filename,
+					     (guchar*) text,
+					     strlen (text),
+					     &result_uncertain);
+	if (result_uncertain)
 	{
-		gchar *curdir, *path;
-		curdir = g_get_current_dir ();
-		path = g_strconcat (curdir, "/", filename, NULL);
-		g_free (curdir);
-		uri = gnome_vfs_get_uri_from_local_path (path);
-		g_free (path);
+		g_free (content_type);
+		content_type = NULL;
 	}
 
-	if ((mime_type = gnome_vfs_get_mime_type (uri)))
-		language = get_language_for_mime_type (mime_type);
+	manager = gtk_source_language_manager_get_default ();
+	language = gtk_source_language_manager_guess_language (manager,
+							       filename,
+							       content_type);
 
-	g_free (mime_type);
-	g_free (uri);
-#endif
-
-	if (!language)
-		language = get_language_for_filename (filename);
+	g_message ("Detected '%s' mime type for file %s, chose language %s",
+		   content_type ? content_type : "(null)",
+		   filename,
+		   language ? gtk_source_language_get_id (language) : "(none)");
 
+	g_free (content_type);
+	g_free (text);
 	return language;
 }
 
@@ -445,7 +375,7 @@ get_language (GtkTextBuffer *buffer, const gchar *filename)
 	GtkSourceLanguage *language = NULL;
 	GtkTextIter start, end;
 	gchar *text;
-	const gchar *lang_string;
+	gchar *lang_string;
 
 	gtk_text_buffer_get_start_iter (buffer, &start);
 	end = start;
@@ -470,7 +400,7 @@ get_language (GtkTextBuffer *buffer, const gchar *filename)
 	}
 
 	if (!language)
-		language = get_language_for_file (filename);
+		language = get_language_for_file (buffer, filename);
 
 	g_free (text);
 	return language;
@@ -574,6 +504,17 @@ margin_toggled_cb (GtkAction *action, gpointer user_data)
 		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
+static void
+hl_bracket_toggled_cb (GtkAction *action, gpointer user_data)
+{
+	GtkTextBuffer *buffer;
+	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_IS_SOURCE_VIEW (user_data));
+	buffer = gtk_text_view_get_buffer (user_data);
+	gtk_source_buffer_set_highlight_matching_brackets (
+		GTK_SOURCE_BUFFER (buffer),
+		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
+}
+
 static void
 hl_line_toggled_cb (GtkAction *action, gpointer user_data)
 {
@@ -583,6 +524,22 @@ hl_line_toggled_cb (GtkAction *action, gpointer user_data)
 		gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
 }
 
+static void
+draw_spaces_toggled_cb (GtkAction *action, gpointer user_data)
+{
+	gboolean draw_spaces;
+
+	g_return_if_fail (GTK_IS_TOGGLE_ACTION (action) && GTK_IS_SOURCE_VIEW (user_data));
+	draw_spaces = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
+
+	if (draw_spaces)
+		gtk_source_view_set_draw_spaces (GTK_SOURCE_VIEW (user_data),
+						 GTK_SOURCE_DRAW_SPACES_ALL);
+	else
+		gtk_source_view_set_draw_spaces (GTK_SOURCE_VIEW (user_data),
+						 0);
+}
+
 static void
 wrap_lines_toggled_cb (GtkAction *action, gpointer user_data)
 {
@@ -665,21 +622,28 @@ new_view_cb (GtkAction *action, gpointer user_data)
 
 /* Buffer action callbacks ------------------------------------------------------------ */
 
+static struct {
+	char *what;
+	char *replacement;
+	GtkSourceSearchFlags flags;
+} search_data = {
+	NULL,
+	NULL,
+	GTK_SOURCE_SEARCH_CASE_INSENSITIVE
+};
+
 static gboolean
-replace_dialog (GtkWidget *widget,
-		char     **what_p,
-		char     **replacement_p)
+search_dialog (GtkWidget            *widget,
+	       gboolean              replace,
+	       char                **what_p,
+	       char                **replacement_p,
+	       GtkSourceSearchFlags *flags_p)
 {
 	GtkWidget *dialog;
 	GtkEntry *entry1, *entry2;
-	static char *what, *replacement;
+	GtkToggleButton *case_sensitive;
 
-	if (!what)
-		what = g_strdup ("gtk");
-	if (!replacement)
-		replacement = g_strdup ("boo");
-
-	dialog = gtk_dialog_new_with_buttons ("Replace",
+	dialog = gtk_dialog_new_with_buttons (replace ? "Replace" : "Find",
 					      GTK_WINDOW (gtk_widget_get_toplevel (widget)),
 					      GTK_DIALOG_MODAL,
 					      GTK_STOCK_CANCEL,
@@ -691,19 +655,27 @@ replace_dialog (GtkWidget *widget,
 
 	entry1 = g_object_new (GTK_TYPE_ENTRY,
 			       "visible", TRUE,
-			       "text", what ? what : "",
+			       "text", search_data.what ? search_data.what : "",
 			       "activates-default", TRUE,
 			       NULL);
 	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 				     GTK_WIDGET (entry1));
 	entry2 = g_object_new (GTK_TYPE_ENTRY,
-			       "visible", TRUE,
-			       "text", replacement ? replacement : "",
+			       "visible", replace,
+			       "text", search_data.replacement ? search_data.replacement : "",
 			       "activates-default", TRUE,
 			       NULL);
 	gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
 				     GTK_WIDGET (entry2));
 
+	case_sensitive = g_object_new (GTK_TYPE_CHECK_BUTTON,
+				       "visible", TRUE,
+				       "label", "Case sensitive",
+				       "active", !(search_data.flags & GTK_SOURCE_SEARCH_CASE_INSENSITIVE),
+				       NULL);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+			    GTK_WIDGET (case_sensitive), FALSE, FALSE, 0);
+
 	while (TRUE)
 	{
 		if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_OK)
@@ -716,47 +688,84 @@ replace_dialog (GtkWidget *widget,
 			break;
 	}
 
-	g_free (what);
-	*what_p = what = g_strdup (gtk_entry_get_text (entry1));
-	g_free (replacement);
-	*replacement_p = replacement = g_strdup (gtk_entry_get_text (entry2));
+	g_free (search_data.what);
+	*what_p = search_data.what = g_strdup (gtk_entry_get_text (entry1));
+	g_free (search_data.replacement);
+	*replacement_p = search_data.replacement = g_strdup (gtk_entry_get_text (entry2));
+	*flags_p = search_data.flags = gtk_toggle_button_get_active (case_sensitive) ?
+					0 : GTK_SOURCE_SEARCH_CASE_INSENSITIVE;
 
 	gtk_widget_destroy (dialog);
 	return TRUE;
 }
 
 static void
-debug_thing_3_cb (GtkAction *action,
-		  gpointer   user_data)
+do_search_replace (GtkTextView *view,
+		   gboolean     replace)
 {
-	GtkTextView *view = user_data;
 	GtkTextBuffer *buffer = gtk_text_view_get_buffer (view);
 	GtkTextIter iter;
 	char *what, *replacement;
+	GtkSourceSearchFlags flags;
 
-	if (!replace_dialog (GTK_WIDGET (view), &what, &replacement))
+	if (!search_dialog (GTK_WIDGET (view), replace, &what, &replacement, &flags))
 		return;
 
-	gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
+	if (replace)
+	{
+		gtk_text_buffer_get_iter_at_offset (buffer, &iter, 0);
 
-	while (TRUE)
+		while (TRUE)
+		{
+			GtkTextIter match_start, match_end;
+
+			if (!gtk_source_iter_forward_search (&iter, what, flags,
+							     &match_start,
+							     &match_end,
+							     NULL))
+			{
+				break;
+			}
+
+			gtk_text_buffer_delete (buffer, &match_start, &match_end);
+			gtk_text_buffer_insert (buffer, &match_start, replacement, -1);
+			iter = match_start;
+		}
+	}
+	else
 	{
 		GtkTextIter match_start, match_end;
 
-		if (!gtk_text_iter_forward_search (&iter, what, 0,
-						   &match_start,
-						   &match_end,
-						   NULL))
-		{
-			break;
-		}
+		gtk_text_buffer_get_iter_at_mark (buffer, &iter, gtk_text_buffer_get_insert (buffer));
 
-		gtk_text_buffer_delete (buffer, &match_start, &match_end);
-		gtk_text_buffer_insert (buffer, &match_start, replacement, -1);
-		iter = match_start;
+		if (gtk_source_iter_forward_search (&iter, what, flags, &match_start, &match_end, NULL))
+		{
+			gtk_text_buffer_select_range (buffer, &match_start, &match_end);
+		}
+		else
+		{
+			GtkTextIter insert = iter;
+			gtk_text_buffer_get_start_iter (buffer, &iter);
+			if (gtk_source_iter_forward_search (&iter, what, flags, &match_start, &match_end, &insert))
+				gtk_text_buffer_select_range (buffer, &match_start, &match_end);
+		}
 	}
 }
 
+static void
+find_cb (GtkAction *action,
+	 gpointer   user_data)
+{
+	do_search_replace (user_data, FALSE);
+}
+
+static void
+replace_cb (GtkAction *action,
+	    gpointer   user_data)
+{
+	do_search_replace (user_data, TRUE);
+}
+
 static void
 open_file_cb (GtkAction *action, gpointer user_data)
 {
@@ -805,12 +814,12 @@ open_file_cb (GtkAction *action, gpointer user_data)
 #ifndef NON_BLOCKING_PAGINATION
 
 static void
-begin_print (GtkPrintOperation        *operation, 
+begin_print (GtkPrintOperation        *operation,
 	     GtkPrintContext          *context,
 	     GtkSourcePrintCompositor *compositor)
 {
 	gint n_pages;
-	
+
 	while (!gtk_source_print_compositor_paginate (compositor, context))
 		;
 
@@ -821,27 +830,27 @@ begin_print (GtkPrintOperation        *operation,
 #else
 
 static gboolean
-paginate (GtkPrintOperation        *operation, 
+paginate (GtkPrintOperation        *operation,
 	  GtkPrintContext          *context,
 	  GtkSourcePrintCompositor *compositor)
 {
 	g_print ("Pagination progress: %.2f %%\n", gtk_source_print_compositor_get_pagination_progress (compositor) * 100.0);
-	
+
 	if (gtk_source_print_compositor_paginate (compositor, context))
 	{
 		gint n_pages;
 
 		g_assert (gtk_source_print_compositor_get_pagination_progress (compositor) == 1.0);
 		g_print ("Pagination progress: %.2f %%\n", gtk_source_print_compositor_get_pagination_progress (compositor) * 100.0);
-		        
+
 		n_pages = gtk_source_print_compositor_get_n_pages (compositor);
 		gtk_print_operation_set_n_pages (operation, n_pages);
 
 
-		
+
 		return TRUE;
 	}
-     
+
 	return FALSE;
 }
 
@@ -865,11 +874,11 @@ draw_page (GtkPrintOperation        *operation,
 	PangoFontDescription *desc;
 	PangoRectangle rect;
 
-	
+
 	cr = gtk_print_context_get_cairo_context (context);
-	
+
 	cairo_save (cr);
-	
+
 	layout = gtk_print_context_create_pango_layout (context);
 
 	pango_layout_set_text (layout, "Draft", -1);
@@ -878,15 +887,15 @@ draw_page (GtkPrintOperation        *operation,
 	pango_layout_set_font_description (layout, desc);
 	pango_font_description_free (desc);
 
-	
+
 	pango_layout_get_extents (layout, NULL, &rect);
-		
-  	cairo_move_to (cr, 
+
+  	cairo_move_to (cr,
   		       (gtk_print_context_get_width (context) - ((double) rect.width / (double) PANGO_SCALE)) / 2,
   		       (gtk_print_context_get_height (context) - ((double) rect.height / (double) PANGO_SCALE)) / 2);
-	  		       
+
 	pango_cairo_layout_path (cr, layout);
-	
+
   	/* Font Outline */
 	cairo_set_source_rgba (cr, 0.85, 0.85, 0.85, 0.80);
 	cairo_set_line_width (cr, 0.5);
@@ -895,7 +904,7 @@ draw_page (GtkPrintOperation        *operation,
 	/* Font Fill */
 	cairo_set_source_rgba (cr, 0.8, 0.8, 0.8, 0.60);
 	cairo_fill (cr);
-	  	  
+
 	g_object_unref (layout);
 	cairo_restore (cr);
 #endif
@@ -905,7 +914,7 @@ draw_page (GtkPrintOperation        *operation,
 }
 
 static void
-end_print (GtkPrintOperation        *operation, 
+end_print (GtkPrintOperation        *operation,
 	   GtkPrintContext          *context,
 	   GtkSourcePrintCompositor *compositor)
 {
@@ -916,12 +925,12 @@ end_print (GtkPrintOperation        *operation,
 #define HEADER_FONT_NAME	"Sans 11"
 #define FOOTER_FONT_NAME	"Sans 11"
 #define BODY_FONT_NAME		"Monospace 9"
-	
+
 /*
-#define SETUP_FROM_VIEW 
+#define SETUP_FROM_VIEW
 */
 
-#undef SETUP_FROM_VIEW 
+#undef SETUP_FROM_VIEW
 
 
 static void
@@ -930,23 +939,23 @@ print_file_cb (GtkAction *action, gpointer user_data)
 	GtkSourceView *view;
 	GtkSourceBuffer *buffer;
 	GtkSourcePrintCompositor *compositor;
-	GtkPrintOperation *operation;	
+	GtkPrintOperation *operation;
 	const gchar *filename;
 	gchar *basename;
-	
+
 	g_return_if_fail (GTK_IS_SOURCE_VIEW (user_data));
-	
+
 	view = GTK_SOURCE_VIEW (user_data);
 
-	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view))); 
-	
-	filename = g_object_get_data (G_OBJECT (buffer), "filename");	
+	buffer = GTK_SOURCE_BUFFER (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
+
+	filename = g_object_get_data (G_OBJECT (buffer), "filename");
 	basename = g_filename_display_basename (filename);
-	
+
 #ifdef SETUP_FROM_VIEW
 	compositor = gtk_source_print_compositor_new_from_view (view);
 #else
-	
+
 	compositor = gtk_source_print_compositor_new (buffer);
 
 	gtk_source_print_compositor_set_tab_width (compositor,
@@ -984,26 +993,26 @@ print_file_cb (GtkAction *action, gpointer user_data)
 
 	gtk_source_print_compositor_set_footer_font_name (compositor,
 							  FOOTER_FONT_NAME);
-#endif	
+#endif
 	operation = gtk_print_operation_new ();
-			
+
 	gtk_print_operation_set_job_name (operation, basename);
-	
+
 	gtk_print_operation_set_show_progress (operation, TRUE);
 
-#ifndef NON_BLOCKING_PAGINATION	
-  	g_signal_connect (G_OBJECT (operation), "begin-print", 
+#ifndef NON_BLOCKING_PAGINATION
+  	g_signal_connect (G_OBJECT (operation), "begin-print",
 			  G_CALLBACK (begin_print), compositor);
 #else
-  	g_signal_connect (G_OBJECT (operation), "paginate", 
+  	g_signal_connect (G_OBJECT (operation), "paginate",
 			  G_CALLBACK (paginate), compositor);
 #endif
-	g_signal_connect (G_OBJECT (operation), "draw-page", 
+	g_signal_connect (G_OBJECT (operation), "draw-page",
 			  G_CALLBACK (draw_page), compositor);
-	g_signal_connect (G_OBJECT (operation), "end-print", 
+	g_signal_connect (G_OBJECT (operation), "end-print",
 			  G_CALLBACK (end_print), compositor);
 
-	gtk_print_operation_run (operation, 
+	gtk_print_operation_run (operation,
 				 GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
 				 NULL, NULL);
 
@@ -1313,8 +1322,11 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 
 	/* add source mark pixbufs */
 	error = NULL;
-	if ((pixbuf = gdk_pixbuf_new_from_file (DATADIR "/pixmaps/apple-green.png", &error)))
+	if ((pixbuf = gdk_pixbuf_new_from_file (TOP_SRCDIR "/tests/gnome-gmush.png", &error)))
 	{
+		GdkColor color;
+		gdk_color_parse ("lightgreen", &color);
+		gtk_source_view_set_mark_category_background (GTK_SOURCE_VIEW (view), MARK_TYPE_1, &color);
 		gtk_source_view_set_mark_category_pixbuf (GTK_SOURCE_VIEW (view), MARK_TYPE_1, pixbuf);
 		gtk_source_view_set_mark_category_priority (GTK_SOURCE_VIEW (view), MARK_TYPE_1, 1);
 		g_object_unref (pixbuf);
@@ -1327,8 +1339,11 @@ create_view_window (GtkSourceBuffer *buffer, GtkSourceView *from)
 	}
 
 	error = NULL;
-	if ((pixbuf = gdk_pixbuf_new_from_file (DATADIR "/pixmaps/apple-red.png", &error)))
+	if ((pixbuf = gdk_pixbuf_new_from_file (TOP_SRCDIR "/tests/apple-red.png", &error)))
 	{
+		GdkColor color;
+		gdk_color_parse ("pink", &color);
+		gtk_source_view_set_mark_category_background (GTK_SOURCE_VIEW (view), MARK_TYPE_2, &color);
 		gtk_source_view_set_mark_category_pixbuf (GTK_SOURCE_VIEW (view), MARK_TYPE_2, pixbuf);
 		gtk_source_view_set_mark_category_priority (GTK_SOURCE_VIEW (view), MARK_TYPE_2, 2);
 		g_object_unref (pixbuf);
@@ -1379,6 +1394,9 @@ create_main_window (GtkSourceBuffer *buffer)
 	/* retrieve the view action group at position 0 in the list */
 	action_group = g_list_nth_data (groups, 0);
 
+	action = gtk_action_group_get_action (action_group, "HlBracket");
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+
 	action = gtk_action_group_get_action (action_group, "ShowNumbers");
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
@@ -1503,12 +1521,10 @@ main (int argc, char *argv[])
 
 	GOptionEntry entries[] = {
 	  { "style-scheme", 's', 0, G_OPTION_ARG_STRING, &style_scheme_id, "Style scheme name to use", "SCHEME"},
-	  { "default-paths", 'd', 0, G_OPTION_ARG_NONE, &use_default_paths, "Style scheme name to use", "SCHEME"},
+	  { "default-paths", 'd', 0, G_OPTION_ARG_NONE, &use_default_paths, "Use default search paths", NULL},
 	  { NULL }
 	};
 
-	g_thread_init (NULL);
-
 #ifdef TEST_XML_MEM
 	init_mem_stuff ();
 #endif
@@ -1520,10 +1536,6 @@ main (int argc, char *argv[])
 
 // 	gdk_window_set_debug_updates (TRUE);
 
-#ifdef USE_GNOME_VFS
-	gnome_vfs_init ();
-#endif
-
 	/* we do not use defaults so we don't need to install the library */
 	lang_dirs = use_default_paths ? NULL : builtin_lang_dirs;
 	lm = gtk_source_language_manager_get_default ();
@@ -1593,9 +1605,5 @@ main (int argc, char *argv[])
 
 	g_free (style_scheme_id);
 
-#ifdef USE_GNOME_VFS
-	gnome_vfs_shutdown ();
-#endif
-
 	return 0;
 }
